@@ -6,6 +6,8 @@ from . import util, defs, instructions
 import textwrap
 from decimal import Decimal, getcontext
 
+logger = logging.getLogger(__name__)
+
 
 def _cfg_satellite():
     """Configure satellite covering the user"""
@@ -15,7 +17,7 @@ def _cfg_satellite():
     help_msg = "Not sure? Check the coverage map at:\n" \
                "https://blockstream.com/satellite/#satellite_network-coverage"
 
-    question = "Please, inform which satellite below covers your location:"
+    question = "Which satellite below covers your location?"
     sat = util._ask_multiple_choice(defs.satellites,
                                     question,
                                     "Satellite",
@@ -25,23 +27,67 @@ def _cfg_satellite():
     return sat
 
 
-def _get_rx_name(rx):
-    """Concatenate vendor/model to form the receiver name"""
-    return (rx['vendor'] + " " + rx['model']).strip()
+def _get_rx_marketing_name(rx):
+    """Get the marketed receiver name (including satellite kit name)"""
+    model = (rx['vendor'] + " " + rx['model']).strip()
+    if model == "Novra S400":
+        model += " (pro kit)"  # append
+    elif model == "TBS 5927":
+        model += " (basic kit)"  # append
+    elif model == "Selfsat IP22":
+        model = "Blockstream Base Station"  # overwrite
+    elif model == "RTL-SDR":
+        model += " software-defined"
+    return model
+
+
+def _get_antenna_name(x):
+    """Get antenna name for the configuration menu"""
+    if x['type'] == 'dish':
+        s = 'Satellite Dish ({})'.format(x['label'])
+    elif x['type'] == 'sat-ip':
+        s = 'Blockstream Base Station (Legacy Output)'
+    else:
+        s = 'Blockstream Flat-Panel Antenna'
+    return s
+
+
+def _ask_antenna():
+    """Configure antenna"""
+    os.system('clear')
+    util._print_header("Antenna")
+
+    question = "What kind of antenna are you using?"
+    antenna = util._ask_multiple_choice(
+        defs.antennas,
+        question,
+        "Antenna",
+        _get_antenna_name,
+        none_option=True,
+        none_str="Other")
+
+    if (antenna is None):
+        size = util.typed_input("Enter size in cm",
+                                "Please enter an integer number in cm",
+                                in_type=int)
+        antenna = {'label': "custom", 'type': 'dish', 'size': size}
+
+    return antenna
 
 
 def _cfg_rx_setup():
     """Configure Rx setup - which receiver user is using """
     os.system('clear')
-    util._print_header("Receiver Setup")
+    util._print_header("Receiver")
 
     # Receiver
-    question = "Please, inform your DVB-S2 receiver setup from the list below:"
+    question = "Select your DVB-S2 receiver:"
     setup = util._ask_multiple_choice(
         defs.demods,
         question,
-        "Setup",
-        lambda x : '{} ({} receiver)'.format(_get_rx_name(x), x['type']))
+        "Receiver",
+        lambda x : '{} receiver'.format(_get_rx_marketing_name(x),
+                                        x['type']))
 
     # Network interface connected to the standalone receiver
     if (setup['type'] == defs.standalone_setup_type):
@@ -51,8 +97,7 @@ def _cfg_rx_setup():
             devices = None
             pass
 
-        question = "Which network interface is connected to the {}?".format(
-            _get_rx_name(setup))
+        question = "Which network interface is connected to the receiver?"
         if (devices is not None):
             netdev = util._ask_multiple_choice(devices,
                                                question,
@@ -63,23 +108,21 @@ def _cfg_rx_setup():
 
         setup['netdev'] = netdev.strip()
 
+    # Define the Sat-IP antenna directly without asking the user
+    if (setup['type'] == defs.sat_ip_setup_type):
+        # NOTE: this works as long as there is a single Sat-IP antenna in the
+        # list. In the future, if other Sat-IP antennas are included, change to
+        # a multiple-choice prompt.
+        for antenna in defs.antennas:
+            if antenna['type'] == 'sat-ip':
+                setup['antenna'] = antenna
+                return setup
+        # We should have returned in the preceding line.
+        raise RuntimeError("Failed to find antenna of {} receiver".format(
+            defs.sat_ip_setup_type))
+
     # Antenna
-    question = "Please, inform the type of your satellite dish (antenna):"
-    size = util._ask_multiple_choice(
-        defs.antennas, question, "Size",
-        lambda x : 'Satellite Dish ({})'.format(x['label'])
-        if x['type'] == 'dish'
-        else '{} Flat-Panel Antenna'.format(x['label']),
-        none_option = True,
-        none_str = "Other")
-
-    if (size is None):
-        resp = util.typed_input("Enter size in cm",
-                                "Please enter an integer number in cm",
-                                in_type=int)
-        size = { 'label' : "custom", 'size' : resp }
-
-    setup['antenna'] = size
+    setup['antenna'] = _ask_antenna()
 
     return setup
 
@@ -159,7 +202,7 @@ def _cfg_custom_lnb(sat):
 
     """
 
-    print("\nPlease inform the specifications of your LNB:")
+    print("\nPlease, inform the specifications of your LNB:")
 
     bands           = ["C", "Ku"]
     question        = "Frequency band:"
@@ -263,16 +306,35 @@ def _ask_lnb(sat):
     return lnb
 
 
+def _ask_psu_voltage(question):
+    """Prompt for the power inserter model or voltage"""
+    psu = util._ask_multiple_choice(
+        defs.psus, question, "Power inserter",
+        lambda x : "{}".format(x['model']),
+        none_option = True,
+        none_str = "No - another model")
+    if (psu is None):
+        voltage = util.typed_input("What is the voltage supplied to the "
+                                   "LNB by your power inserter?")
+    else:
+        voltage = psu["voltage"]
+
+    return voltage
+
+
 def _cfg_lnb(sat, setup):
     """Configure LNB - either from preset or from custom specs
 
+    Configure also the LNB power supply, if applicable.
+
     Args:
         sat   : user's satellite info
         setup : user's setup info
 
     """
 
-    if (setup['antenna']['type'] == 'flat'):
+    # For flat-panel and Sat-IP antennas, the LNB is the integrated one
+    if (setup['antenna']['type'] in ['flat', 'sat-ip']):
         for lnb in defs.lnbs:
             if lnb['vendor'] == 'Selfsat':
                 lnb["v1_pointed"] = False
@@ -297,35 +359,34 @@ def _cfg_lnb(sat, setup):
                                                             sat['alias']))
         sys.exit(1)
 
-    # For dual polarization LNBs, we must know whether it was pointed before for
-    # blocksat v1 in order to define the polarization on channels.conf
-    if (lnb['pol'].lower() == "dual"):
-        print()
-        prev_setup = util._ask_yes_or_no(
+    # When configuring a non-SDR receiver with a dual polarization LNB, we must
+    # know whether the LNB was pointed before on an SDR setup in order to define
+    # the polarization on channels.conf. When configuring an SDR receiver,
+    # collect the power inserter voltage too for future use.
+    if ((lnb['pol'].lower() == "dual" and setup['type'] != defs.sdr_setup_type)
+        or setup['type'] == defs.sdr_setup_type):
+        os.system('clear')
+        util._print_header("Power Supply")
+
+    if (lnb['pol'].lower() == "dual" and setup['type'] != defs.sdr_setup_type):
+        prev_sdr_setup = util._ask_yes_or_no(
             "Are you reusing an LNB that is already pointed and that was used "
-            "for the previous version of Blockstream Satellite (before the "
-            "upgrade to DVB-S2)?",
-        help_msg="NOTE: this information is helpful to determine the "
-            "polarization required for the LNB.")
+            "before by an SDR receiver?",
+            default='n',
+            help_msg="NOTE: this information is helpful to determine the "
+            "polarization required for the LNB."
+        )
 
-        if (prev_setup):
-            question = ("In this setup, did you use one of the LNB power "
-                        "inserters below?")
-            psu = util._ask_multiple_choice(
-                defs.psus, question, "Power inserter",
-                lambda x : "{}".format(x['model']),
-                none_option = True,
-                none_str = "No - another model")
-            if (psu is None):
-                voltage = util.typed_input("What is the voltage supplied to the "
-                                           "LNB by your power inserter?")
-            else:
-                voltage = psu["voltage"]
+        lnb["v1_pointed"] = prev_sdr_setup
 
-            lnb["v1_pointed"]     = True
-            lnb["v1_psu_voltage"] = voltage
-        else:
-            lnb["v1_pointed"] = False
+        if (prev_sdr_setup):
+            print()
+            question = ("In the pre-existing SDR setup, did you use one of the "
+                        "LNB power inserters below?")
+            lnb["v1_psu_voltage"] = _ask_psu_voltage(question)
+    elif (setup['type'] == defs.sdr_setup_type):
+        question = ("Are you using one of the LNB power inserters below?")
+        lnb["psu_voltage"] = _ask_psu_voltage(question)
 
     return lnb
 
@@ -368,7 +429,7 @@ def _cfg_frequencies(sat, lnb, setup):
     if (if_freq < setup['tun_range'][0] or if_freq > setup['tun_range'][1]):
         logging.error("Your LNB yields an L-band frequency that is out of "
                       "the tuning range of the {} receiver.".format(
-                          _get_rx_name(setup)))
+                          _get_rx_marketing_name(setup)))
         sys.exit(1)
 
     return {
@@ -381,7 +442,7 @@ def _cfg_frequencies(sat, lnb, setup):
 def _cfg_chan_conf(info, chan_file):
     """Generate the channels.conf file"""
 
-    util._print_header("Channel Configurations for Linux USB Rx")
+    util._print_header("Channel Configuration")
 
     print(textwrap.fill("This step will generate the channel configuration "
                         "file that is required when launching the USB "
@@ -400,7 +461,8 @@ def _cfg_chan_conf(info, chan_file):
         f.write('[blocksat-ch]\n')
         f.write('\tDELIVERY_SYSTEM = DVBS2\n')
         f.write('\tFREQUENCY = %u\n' %(int(info['sat']['dl_freq']*1000)))
-        if (info['lnb']['pol'].lower() == "dual" and info['lnb']['v1_pointed']):
+        if (info['lnb']['pol'].lower() == "dual" and 'v1_pointed' in info['lnb']
+            and info['lnb']['v1_pointed']):
             # If a dual-polarization LNB is already pointed for Blocksat v1,
             # then we must use the polarization that the LNB was pointed to
             # originally, regardless of the satellite signal's polarization. In
@@ -423,7 +485,8 @@ def _cfg_chan_conf(info, chan_file):
             defs.sym_rate[info['sat']['alias']]))
         f.write('\tINVERSION = AUTO\n')
         f.write('\tMODULATION = QPSK\n')
-        f.write('\tVIDEO_PID = 32+33\n')
+        pids = "+".join([str(x) for x in defs.pids])
+        f.write('\tVIDEO_PID = {}\n'.format(pids))
 
     print("File \"%s\" saved." %(chan_file))
 
@@ -431,6 +494,17 @@ def _cfg_chan_conf(info, chan_file):
         logging.debug(f.read())
 
 
+def _parse_chan_conf(chan_file):
+    """Convert channel.conf file contents to dictionary"""
+    chan_conf = {}
+    with open(chan_file, 'r') as f:
+        lines = f.read().splitlines()
+        for line in lines[1:]:
+            key, val = line.split('=')
+            chan_conf[key.strip()] = val.strip()
+    return chan_conf
+
+
 def _cfg_file_name(cfg_name, directory):
     """Get the name of the configuration JSON file"""
     # Remove paths
@@ -451,6 +525,12 @@ def _read_cfg_file(cfg_file):
         return info
 
 
+def _write_cfg_file(cfg_file, user_info):
+    """Write configuration file"""
+    with open(cfg_file, 'w') as fd:
+        json.dump(user_info, fd)
+
+
 def _rst_cfg_file(cfg_file):
     """Reset a previous configuration file in case it exists"""
     info = _read_cfg_file(cfg_file)
@@ -489,6 +569,82 @@ def read_cfg_file(cfg_name, directory):
     return info
 
 
+def write_cfg_file(cfg_name, directory, user_info):
+    """Write configuration file"""
+    cfg_file = _cfg_file_name(cfg_name, directory)
+    _write_cfg_file(cfg_file, user_info)
+
+
+def get_rx_model(user_info):
+    """Return string with the receiver vendor and model
+
+    Note: this function differs from _get_rx_marketing_name(). The latter
+    includes the satellite kit name. In contrast, this function returns the raw
+    vendeor-model string.
+
+    """
+    return (user_info['setup']['vendor'] + " " + \
+        user_info['setup']['model']).strip()
+
+
+def get_antenna_model(user_info):
+    """Return string with the antenna model"""
+    antenna_info = user_info['setup']['antenna']
+    antenna_type = antenna_info.get('type')
+
+    if (antenna_type == 'dish' or antenna_info['label'] == 'custom'):
+        dish_size = antenna_info['size']
+        if (dish_size >= 100):
+            antenna = "{:g}m dish".format(dish_size / 100)
+        else:
+            antenna = "{:g}cm dish".format(dish_size)
+    else:
+        return antenna_info['label']
+
+    return antenna
+
+
+def get_lnb_model(user_info):
+    """Return string with the LNB model"""
+    lnb_info = user_info['lnb']
+    if (lnb_info['vendor'] == "" and lnb_info['model'] == ""):
+        if (lnb_info['universal']):
+            lnb = "Custom Universal LNB"
+        else:
+            lnb = "Custom {}-band LNB".format(
+                lnb_info['band'])
+    else:
+        lnb = "{} {}".format(lnb_info['vendor'], lnb_info['model'])
+    return lnb
+
+
+def get_net_if(user_info):
+    """Get the network interface used by the given setup"""
+    if (user_info is None):
+        raise ValueError("Failed to read local configuration")
+
+    setup_type = user_info['setup']['type']
+
+    if (setup_type in [defs.sdr_setup_type, defs.sat_ip_setup_type]):
+        interface = "lo"
+    elif (setup_type == defs.linux_usb_setup_type):
+        if ('adapter' not in user_info['setup']):
+            interface = "dvb0_0"
+            logger.warning("Could not find the dvbnet interface name. "
+                           "Is the {} receiver running?".format(
+                               get_rx_model(user_info)
+                           ))
+            logger.warning("Assuming interface name {}.".format(interface))
+        else:
+            adapter = user_info['setup']['adapter']
+            interface = "dvb{}_0".format(adapter)
+    elif (setup_type == defs.standalone_setup_type):
+        interface = user_info['setup']['netdev']
+    else:
+        raise ValueError("Unknown setup type")
+    return interface
+
+
 def subparser(subparsers):
     """Argument parser of config command"""
     p = subparsers.add_parser('config', aliases=['cfg'],
@@ -496,10 +652,23 @@ def subparser(subparsers):
                               help='Define receiver and Bitcoin Satellite \
                               configurations',
                               formatter_class=ArgumentDefaultsHelpFormatter)
-    p.add_argument('-c', '--chan-conf',
-                   default="channels.conf",
-                   help='Channel configurations file')
     p.set_defaults(func=configure)
+
+    subsubparsers = p.add_subparsers(title='subcommands',
+                                     help='Target sub-command')
+    p2 = subsubparsers.add_parser(
+        'show',
+        description="Display the local configuration",
+        help='Display the local configuration',
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    p2.set_defaults(func=show)
+
+    p3 = subsubparsers.add_parser(
+        'channel',
+        description="Configure the channels file used by the USB receiver",
+        help='Configure the channels file used by the USB receiver',
+        formatter_class=ArgumentDefaultsHelpFormatter)
+    p3.set_defaults(func=channel)
     return p
 
 
@@ -539,10 +708,10 @@ def configure(args):
         _cfg_chan_conf(user_info, chan_file)
         user_info['setup']['channel'] = chan_file
 
-    # JSON configuration file
-    with open(cfg_file, 'w') as fd:
-        json.dump(user_info, fd)
+    # Create the JSON configuration file
+    _write_cfg_file(cfg_file, user_info)
 
+    os.system('clear')
     util._print_header("JSON configuration file")
     print("Saved configurations on %s" %(cfg_file))
 
@@ -555,3 +724,54 @@ def configure(args):
     """)
 
 
+def show(args):
+    """Print the local configuration"""
+    info = read_cfg_file(args.cfg, args.cfg_dir)
+    if (info is None):
+        return
+    print("| {:30s} | {:25s} |".format("Receiver",
+                                       _get_rx_marketing_name(info['setup'])))
+    print("| {:30s} | {:25s} |".format("LNB", get_lnb_model(info)))
+    print("| {:30s} | {:25s} |".format("Antenna", get_antenna_model(info)))
+    pr_cfgs = {
+        'freqs': {
+            'dl': ("Downlink frequency", "MHz", None),
+            'lo': ("LNB LO frequency", "MHz", None),
+            'l_band': ("Receiver L-band frequency", "MHz", None)
+        },
+        'sat': {
+            'name': ("Satellite name", "", None),
+            'band': ("Signal band", "", None),
+            'pol': ("Signal polarization", "",
+                    lambda x : "Horizontal" if x == "H" else "vertical"),
+        }
+    }
+    for category in pr_cfgs:
+        for key, pr_cfg in pr_cfgs[category].items():
+            label = pr_cfg[0]
+            unit = pr_cfg[1]
+            val = info[category][key]
+            # Transform the value if a callback is defined
+            if (pr_cfg[2] is not None):
+                val = pr_cfg[2](val)
+            print("| {:30s} | {:25s} |".format(label, str(val) + " " + unit))
+
+
+def channel(args):
+    """Configure the channels.conf file directly"""
+    user_info = read_cfg_file(args.cfg, args.cfg_dir)
+    if (user_info is None):
+        return
+
+    # Channel configuration file
+    if (user_info['setup']['type'] != defs.linux_usb_setup_type):
+        raise TypeError("Invalid command for {} receivers".format(
+            user_info['setup']['type']))
+
+    chan_file = os.path.join(args.cfg_dir, args.cfg + "-channel.conf")
+    _cfg_chan_conf(user_info, chan_file)
+
+    # Overwrite the channels.conf path in case it is changing from a previous
+    # version of the CLI when the conf file name was not bound to the cfg name
+    user_info['setup']['channel'] = chan_file
+    write_cfg_file(args.cfg, args.cfg_dir, user_info)
