@@ -1,10 +1,14 @@
 # Constants
-mcast_ip       = "239.0.0.2"
-btc_port       = "4434"
-btc_dst_addr   = mcast_ip + ":" + btc_port
-src_ports      = ["4433", "4434"]
-pids           = [32, 33]
-rolloff        = 0.2
+mcast_ip        = "239.0.0.2"
+api_port        = 4433
+btc_port        = 4434
+monitor_port    = 4435
+api_dst_addr    = mcast_ip + ":" + str(api_port)
+btc_dst_addr    = mcast_ip + ":" + str(btc_port)
+src_ports       = [str(api_port), str(btc_port)]
+pids            = [32]
+rolloff         = 0.2
+pilots          = True
 sym_rate = {
     'G18'      : 1000000,
     'E113'     : 1000000,
@@ -14,9 +18,6 @@ sym_rate = {
     'T18V Ku'  : 1000000
 }
 
-low_rate_modcod  = "0x0010"
-high_rate_modcod = "0x2000"
-
 satellites  = [
     {
         'name'    : "Galaxy 18",
@@ -71,6 +72,7 @@ satellites  = [
 linux_usb_setup_type  = "Linux USB"
 sdr_setup_type        = "Software-defined"
 standalone_setup_type = "Standalone"
+sat_ip_setup_type     = "Sat-IP"
 
 demods = [
     {
@@ -90,6 +92,12 @@ demods = [
         'model'     : "RTL-SDR",
         'type'      : sdr_setup_type,
         'tun_range' : (24.0, 1766.0) # assuming R820T2
+    },
+    {
+        'vendor'    : "Selfsat",
+        'model'     : "IP22",
+        'type'      : sat_ip_setup_type,
+        'tun_range' : (950, 2150)
     }
 ]
 
@@ -102,7 +110,8 @@ antennas = [
     { 'label' : "1.5m / 5ft",   'type': 'dish', 'size' : 150  },
     { 'label' : "1.8m / 6ft",   'type': 'dish', 'size' : 180  },
     { 'label' : "2.4m / 8ft",   'type': 'dish', 'size' : 240  },
-    { 'label' : 'Blockstream',  'type': 'flat', 'size' : None },
+    { 'label' : 'Selfsat-H50D', 'type': 'flat', 'size' : None },
+    { 'label' : 'Selfsat>IP22', 'type': 'sat-ip', 'size' : None }
 ]
 
 ku_band_thresh = 11700.0
@@ -128,7 +137,7 @@ lnbs = [
     },
     {
         'vendor'    : "Selfsat",
-        'model'     : "H50D",
+        'model'     : "Integrated LNB",
         'in_range'  : [10700.0, 12750.0],
         'lo_freq'   : [9750.0, 10600.0],
         'universal' : True,
@@ -275,4 +284,35 @@ v4l_lnbs = [
     }
 ]
 
+modcods = {
+    'qpsk1/4': 1,
+    'qpsk1/3': 2,
+    'qpsk2/5': 3,
+    'qpsk1/2': 4,
+    'qpsk3/5': 5,
+    'qpsk2/3': 6,
+    'qpsk3/4': 7,
+    'qpsk4/5': 8,
+    'qpsk5/6': 9,
+    'qpsk8/9': 10,
+    'qpsk9/10': 11,
+    '8psk3/5': 12,
+    '8psk2/3': 13,
+    '8psk3/4': 14,
+    '8psk5/6': 15,
+    '8psk8/9': 16,
+    '8psk9/10': 17,
+    '16apsk2/3': 18,
+    '16apsk3/4': 19,
+    '16apsk4/5': 20,
+    '16apsk5/6': 21,
+    '16apsk8/9': 22,
+    '16apsk9/10': 23,
+    '32apsk3/4': 24,
+    '32apsk4/5': 25,
+    '32apsk5/6': 26,
+    '32apsk8/9': 27,
+    '32apsk9/10': 28
+}
+
 lnb_options = [x['alias'] for x in v4l_lnbs]
