@@ -5,9 +5,17 @@ import sys, os, subprocess, logging, glob, json
 from shutil import which
 from pprint import pformat
 import platform, distro, requests
+from distutils.version import LooseVersion
 logger = logging.getLogger(__name__)
 
 
+target_map = {
+    "sdr": defs.sdr_setup_type,
+    "usb": defs.linux_usb_setup_type,
+    "standalone": defs.standalone_setup_type,
+    "sat-ip": defs.sat_ip_setup_type
+}
+
 def _download_file(url, destdir, dry_run):
     filename   = url.split('/')[-1]
     local_path = os.path.join(destdir, url.split('/')[-1])
@@ -34,7 +42,8 @@ def _check_distro(supported_distros, setup_type):
     base_url = "https://github.com/Blockstream/satellite/blob/master/doc/"
     instructions_url = {
         defs.sdr_setup_type : "sdr.md",
-        defs.linux_usb_setup_type : "tbs.md"
+        defs.linux_usb_setup_type : "tbs.md",
+        defs.sat_ip_setup_type : "sat-ip.md"
     }
     full_url = base_url + instructions_url[setup_type]
     if (distro.id() not in supported_distros):
@@ -229,93 +238,46 @@ def _install_common(interactive=True, update=False, dry=False, btc=False):
 
 
 
-def _install_sdr(interactive=True, update=False, dry=False):
-    """Install SDR dependencies"""
-    util._print_header("Installing SDR Dependencies")
-    apt_pkg_list = ["gqrx-sdr", "rtl-sdr", "leandvb", "tsduck"]
-    dnf_pkg_list = ["gqrx", "rtl-sdr", "leandvb", "tsduck"]
-    yum_pkg_list = ["rtl-sdr", "leandvb", "tsduck"]
-    # NOTE: leandvb and tsduck come from our repository. Also, note gqrx is not
-    # available on CentOS (hence not included on the yum list).
-    _install_packages(apt_pkg_list, dnf_pkg_list, yum_pkg_list, interactive,
+def _install_specific(target, interactive=True, update=False, dry=False):
+    """Install setup-specific dependencies"""
+    key = next(key for key, val in target_map.items() if val == target)
+    pkg_map = {
+        'sdr': {
+            'apt': ["gqrx-sdr", "rtl-sdr", "leandvb", "tsduck"],
+            'dnf': ["gqrx", "rtl-sdr", "leandvb", "tsduck"],
+            'yum': ["rtl-sdr", "leandvb", "tsduck"]
+        },
+        'usb': {
+            'apt': ["iproute2", "iptables", "dvb-apps", "dvb-tools"],
+            'dnf': ["iproute", "iptables", "dvb-apps", "v4l-utils"],
+            'yum': ["iproute", "iptables", "dvb-apps", "v4l-utils"]
+        },
+        'standalone': {
+            'apt': ["iproute2", "iptables"],
+            'dnf': ["iproute", "iptables"],
+            'yum': ["iproute", "iptables"]
+        },
+        'sat-ip': {
+            'apt': ["tsduck"],
+            'dnf': ["tsduck"],
+            'yum': ["tsduck"]
+        }
+    }
+    # NOTE:
+    # - leandvb and tsduck come from our repository. Also, note gqrx is not
+    #   available on CentOS (hence not included on the yum list).
+    # - In the USB setup, the only package from our repository is dvb-apps in
+    #   the specific case of dnf (RPM) installation, given that dvb-apps is not
+    #   available on the latest mainstream fedora repositories.
+
+    util._print_header("Installing {} Receiver Dependencies".format(target))
+    _install_packages(pkg_map[key]['apt'],
+                      pkg_map[key]['dnf'],
+                      pkg_map[key]['yum'],
+                      interactive,
                       update, dry)
 
 
-def _install_usb(interactive=True, update=False, dry=False):
-    """Install USB receiver dependencies"""
-    util._print_header("Installing USB Receiver Dependencies")
-    apt_pkg_list = ["iproute2", "iptables", "dvb-apps", "dvb-tools"]
-    dnf_pkg_list = ["iproute", "iptables", "dvb-apps", "v4l-utils"]
-    yum_pkg_list = ["iproute", "iptables", "dvb-apps", "v4l-utils"]
-    # NOTE: the only package from our repository is dvb-apps in the specific
-    # case of dnf (RPM) installation, because dvb-apps is not available on the
-    # mainstream fc31/32 repository
-    _install_packages(apt_pkg_list, dnf_pkg_list, yum_pkg_list, interactive,
-                      update, dry)
-
-
-def _install_standalone(interactive=True, update=False, dry=False):
-    """Install standalone receiver dependencies"""
-    util._print_header("Installing Standalone Receiver Dependencies")
-    apt_pkg_list = ["iproute2", "iptables"]
-    dnf_pkg_list = ["iproute", "iptables"]
-    yum_pkg_list = ["iproute", "iptables"]
-    _install_packages(apt_pkg_list, dnf_pkg_list, yum_pkg_list, interactive,
-                      update, dry)
-
-
-def _check_cli_updates(interactive):
-    """Check if the CLI has updates"""
-
-    logger.debug("Checking blocksat-cli updates")
-
-    # If pip is not available, then definitely the CLI was not installed via pip
-    if (not which("pip3")):
-        return
-
-    # Is blocksat-cli installed?
-    res = subprocess.run(["pip3", "show", "blocksat-cli"],
-                         stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
-    found = (res.returncode == 0)
-
-    # If the CLI was not installed via pip, don't check updates
-    if (not found):
-        return
-
-    # Is the CLI outdated?
-    try:
-        outdated_list = json.loads(
-            subprocess.check_output(["pip3", "list", "--outdated",
-                                     "--format", "json"],
-                                    stderr=subprocess.DEVNULL).decode()
-        )
-    except subprocess.CalledProcessError:
-        # Don't break if the command fails. It could be a problem on pip. For
-        # example, there is a bug in which pip throws a TypeError on the "list
-        # --outdated" command saying: '>' not supported between instances of
-        # 'Version' and 'Version'
-        logger.warning("Failed to check blocksat-cli updates.")
-        return
-
-    cli_outdated = False
-    for package in outdated_list:
-        if (package['name'] == "blocksat-cli"):
-            cli_outdated = True
-            print("Update available for blocksat-cli")
-            print("Current version: {}.\nLatest version: {}.".format(
-                package['version'], package['latest_version']
-            ))
-            if ((not interactive) or util._ask_yes_or_no("Update now?")):
-                cmd = ["pip3", "install", "blocksat-cli", "--upgrade"]
-                util.run_and_log(util.root_cmd(cmd))
-                print("Updated successfully")
-                print("Please, run command again to use the new version")
-                exit()
-
-    if (not cli_outdated):
-        logger.debug("blocksat-cli is up-to-date")
-
-
 def _print_help(args):
     """Re-create argparse's help menu"""
     parser     = ArgumentParser()
@@ -349,7 +311,7 @@ def subparser(subparsers):
                             description="Install software dependencies",
                             help='Install software dependencies')
     p1.add_argument("--target",
-                    choices=["sdr", "usb", "standalone"],
+                    choices=list(target_map.keys()),
                     default=None,
                     help="Target setup type for installation of dependencies")
     p1.add_argument("--btc",
@@ -362,7 +324,7 @@ def subparser(subparsers):
                             description="Update software dependencies",
                             help='Update software dependencies')
     p2.add_argument("--target",
-                    choices=["sdr", "usb", "standalone"],
+                    choices=list(target_map.keys()),
                     default=None,
                     help="Target setup type for the update of dependencies")
     p2.add_argument("--btc",
@@ -385,16 +347,7 @@ def run(args):
     # Interactive installation? I.e., requires user to press "y/n"
     interactive = (not args.yes)
 
-    # Check CLI updates
-    if (not args.dry_run):
-        _check_cli_updates(interactive)
-
     if (args.target is not None):
-        target_map = {
-            "sdr"        : defs.sdr_setup_type,
-            "usb"        : defs.linux_usb_setup_type,
-            "standalone" : defs.standalone_setup_type
-        }
         target = target_map[args.target]
     else:
         info = config.read_cfg_file(args.cfg, args.cfg_dir)
@@ -416,30 +369,15 @@ def run(args):
     _install_common(interactive=interactive, update=args.update,
                     dry=args.dry_run, btc=args.btc)
 
-    if (target == defs.sdr_setup_type):
-        # The SDR packages are only available to the distributions below:
-        _check_distro(["ubuntu", "fedora", "centos"],
-                      defs.sdr_setup_type)
-        _install_sdr(interactive=interactive, update=args.update,
-                     dry=args.dry_run)
-    elif (target == defs.linux_usb_setup_type):
-        _install_usb(interactive=interactive, update=args.update,
-                     dry=args.dry_run)
-    elif (target == defs.standalone_setup_type):
-        _install_standalone(interactive=interactive, update=args.update,
-                            dry=args.dry_run)
-    else:
-        raise ValueError("Unexpected receiver target")
+    # Install setup-specific dependencies
+    _install_specific(target, interactive=interactive, update=args.update,
+                      dry=args.dry_run)
 
 
 def drivers(args):
     # Interactive installation? I.e., requires user to press "y/n"
     interactive = (not args.yes)
 
-    # Check CLI updates
-    if (not args.dry_run):
-        _check_cli_updates(interactive)
-
     if (os.geteuid() != 0 and not args.dry_run):
         util.fill_print("Some commands require root access and will prompt "
                         "for password")
@@ -458,8 +396,9 @@ def drivers(args):
                      "libproc-processtable-perl",
                      linux_headers]
     dnf_pkg_list   = ["make", "gcc", "git", "patch", "patchutils",
-                     "perl-Proc-ProcessTable", "perl-Digest-SHA",
-                      kernel_devel, kernel_headers]
+                      "perl-Proc-ProcessTable", "perl-Digest-SHA",
+                      "perl-File-Copy-Recursive", kernel_devel,
+                      kernel_headers]
     yum_pkg_list   =  dnf_pkg_list
 
     # On dnf, not always the kernel-devel/headers package will be available for
@@ -501,7 +440,9 @@ def drivers(args):
                 sys.exit(1)
                 return
         else:
-            logger.warning("Could not find an available kernel update")
+            logger.error("Could not find an available kernel update")
+            sys.exit(1)
+            return
 
     _update_pkg_repo(interactive, args.dry_run)
     _install_packages(apt_pkg_list, dnf_pkg_list, yum_pkg_list,
@@ -538,8 +479,11 @@ def drivers(args):
     runner.run(["make", "allyesconfig"], cwd = media_build_dir)
 
     # FIXME: Temporary workaround for error "modpost: "__devm_regmap_init_sccb"
-    # ov9650.ko undefined!": disable ov9650 from the build.
-    if (distro.id() == "fedora"):
+    # ov9650.ko undefined!": disable ov9650 from the build. The problem was
+    # observed on kernel versions 5.3.7 and 5.7.7. Apply the workaround for any
+    # version < 5.8.
+    if (distro.id() == "fedora" and
+        LooseVersion(linux_release) < LooseVersion('5.8')):
         runner.run(["sed", "-i",
                     "s/CONFIG_VIDEO_OV9650=m/CONFIG_VIDEO_OV9650=n/g",
                     "v4l/.config"], cwd = media_build_dir)
