@@ -1,20 +1,21 @@
 #!/usr/bin/env python3
-import logging, os
+
+
+__version__ = "0.3.2"
+
+
+import logging, os, time
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
 from . import defs, config, util, instructions, gqrx, bitcoin, sdr, rp, \
-    firewall, standalone, usb, dependencies
+    firewall, standalone, usb, satip, dependencies, update
+from .api import api
 from os import environ
 import platform
 
 
-__version__ = "0.2.9"
-
-
 def main():
     """Main - parse command-line arguments and call subcommands
     """
-    assert(platform.system() == 'Linux'), \
-        "blocksat-cli currently only supports Linux"
     sudo_user       = environ.get('SUDO_USER')
     user            = sudo_user if sudo_user is not None else ""
     home            = os.path.expanduser("~" + user)
@@ -31,11 +32,15 @@ def main():
     parser.add_argument('--cfg-dir',
                         default=default_cfg_dir,
                         help="Directory to use for configuration files")
+    parser.add_argument('--utc',
+                        action='store_true',
+                        help="Print log timestamps in UTC")
     parser.add_argument('-v', '--version', action='version',
                         version='%(prog)s {}'.format(__version__))
 
     subparsers = parser.add_subparsers(title='subcommands',
-                                       help='Target sub-command')
+                                       help='Target sub-command',
+                                       dest='subcommand')
 
     config.subparser(subparsers)
     instructions.subparser(subparsers)
@@ -47,14 +52,30 @@ def main():
     gqrx.subparser(subparsers)
     bitcoin.subparser(subparsers)
     sdr.subparser(subparsers)
+    api.subparser(subparsers)
+    satip.subparser(subparsers)
 
     args = parser.parse_args()
-    log_format = '%(levelname)s: %(message)s'
-    if (args.debug):
-        logging.basicConfig(level=logging.DEBUG, format=log_format)
-        logging.debug('[Debug Mode]')
-    else:
-        logging.basicConfig(level=logging.INFO, format=log_format)
+
+    # Filter commands that are Linux-only
+    if (args.subcommand not in ["cfg", "instructions", "api", "btc"]):
+        assert(platform.system() == 'Linux'), \
+            "Command {} is currently Linux-only".format(args.subcommand)
+
+    # Logging
+    logging_fmt = '%(asctime)s %(levelname)-8s %(name)s %(message)s' if \
+                  args.debug else '%(asctime)s %(levelname)-8s %(message)s'
+    logging.basicConfig(
+        level=logging.DEBUG if args.debug else logging.INFO,
+        format=logging_fmt,
+        datefmt='%Y-%m-%d %H:%M:%S'
+    )
+    if (args.utc):
+        logging.Formatter.converter = time.gmtime
+    logging.debug('[Debug Mode]')
+
+    # Check CLI updates
+    update.check_cli_updates(args, __version__)
 
     if hasattr(args, 'func'):
         args.func(args)
