@@ -1,8 +1,8 @@
 """SDR Receiver Wrapper"""
+import subprocess, logging, textwrap, os, threading, time
 from argparse import ArgumentDefaultsHelpFormatter
-from . import config, defs, util, dependencies
-import subprocess, logging, textwrap, os
 from shutil import which
+from . import config, defs, util, dependencies, monitoring, tsp
 logger = logging.getLogger(__name__)
 
 
@@ -36,6 +36,108 @@ def _tune_max_pipe_size(pipesize):
         return True
 
 
+# Rx status metrics printed by leandvb
+rx_stat_map = {
+    'SS' : { # based on the AGC but not calibrated to dBm (unitless)
+        'key'  : 'level',
+        'unit' : None
+    },
+    'MER' : {
+        'key'  : 'snr',
+        'unit' : 'dB'
+    },
+    'VBER' : { # measured based on the BCH decoder output
+        'key'  : 'ber',
+        'unit' : None
+    }
+}
+
+
+def _get_monitor(args):
+    """Create an object of the Monitor class
+
+    Args:
+        args      : SDR parser arguments
+        sat_name  : Satellite name
+
+    """
+    # If debugging leandvb, don't echo the logs to stdout, otherwise the
+    # logs get mixed on the console and it becomes hard to read them.
+    echo = (args.debug_dvbs2 == 0)
+
+    # Force scrolling logs if tsp is configured to print to stdout
+    scrolling = tsp.prints_to_stdout(args) or args.log_scrolling
+
+    return monitoring.Monitor(
+        args.cfg_dir,
+        logfile = args.log_file,
+        scroll = scrolling,
+        echo = echo,
+        min_interval = args.log_interval,
+        server = args.monitoring_server,
+        port = args.monitoring_port,
+        report = args.report,
+        report_opts = monitoring.get_report_opts(args),
+        utc = args.utc
+    )
+
+
+def _monitor_demod_info(info_pipe, monitor):
+    """Monitor leandvb's demodulator information
+
+    Continuously read the demodulator information printed by leandvb into the
+    descriptor pointed by --fd-info, which is tied to an unnamed pipe
+    file. Then, feed the information into the monitor object.
+
+    Args:
+        info_pipe : Pipe object pointing to the pipe file that is used as
+                    leandvb's --fd-info descriptor
+        monitor   : Object of the Monitor class used to handle receiver
+                    monitoring and logging
+
+    """
+    assert(isinstance(info_pipe, util.Pipe))
+
+    # "Standard" status format accepted by the Monitor class
+    status = {
+        'lock': (False, None),
+        'level': None,
+        'snr': None,
+        'ber': None
+    }
+
+    while True:
+        line = info_pipe.readline()
+
+        if "FRAMELOCK" in line:
+            status['lock'] = (line.split()[-1] == "1", None)
+
+        for metric in rx_stat_map:
+            if metric in line:
+                val         = float(line.split()[-1])
+                unit        = rx_stat_map[metric]['unit']
+                key         = rx_stat_map[metric]['key']
+                status[key] = (val, unit)
+
+        # If unlocked, clear the status metrics that depend on receiver locking
+        # (they will show garbage if unlocked). If locked, just make sure that
+        # all the required metrics are filled in the status dictionary.
+        ready = True
+        for metric in rx_stat_map:
+            key = rx_stat_map[metric]['key']
+            if status['lock'][0]:
+                if key not in status or status[key] is None:
+                    ready = False
+            else:
+                if (key in status):
+                    del status[key]
+
+        if (not ready):
+            continue
+
+        monitor.update(status)
+
+
 def subparser(subparsers):
     """Parser for sdr command"""
     p = subparsers.add_parser('sdr',
@@ -43,25 +145,27 @@ def subparser(subparsers):
                               help='Launch SDR receiver',
                               formatter_class=ArgumentDefaultsHelpFormatter)
 
+    # Monitoring options
+    monitoring.add_to_parser(p)
+
     rtl_p = p.add_argument_group('rtl_sdr options')
     rtl_p.add_argument('--sps', default=2.0, type=float,
                        help='Samples per symbol, or, equivalently, the '
                            'target oversampling ratio')
-    rtl_group = rtl_p.add_mutually_exclusive_group()
-    rtl_group.add_argument('--rtl-idx', default=0, type=int,
-                           help='RTL-SDR device index')
-    rtl_group.add_argument('-g', '--gain', default=40, type=float,
-                           help='RTL-SDR Rx gain')
-    rtl_group.add_argument('-f', '--iq-file', default=None,
-                           help='File to read IQ samples from instead of reading '
-                           'from the RTL-SDR in real-time')
+    rtl_p.add_argument('--rtl-idx', default=0, type=int,
+                       help='RTL-SDR device index')
+    rtl_p.add_argument('-g', '--gain', default=40, type=float,
+                       help='RTL-SDR Rx gain')
+    rtl_p.add_argument('-f', '--iq-file', default=None,
+                       help='File to read IQ samples from instead of reading '
+                       'from the RTL-SDR in real-time')
 
     ldvb_p = p.add_argument_group('leandvb options')
     ldvb_p.add_argument('-n', '--n-helpers', default=6, type=int,
                         help='Number of instances of the external LDPC decoder \
                         to spawn as child processes')
-    ldvb_p.add_argument('-d', '--debug-ts', action='count', default=0,
-                        help="Activate debugging on leandvb. Use it multiple "
+    ldvb_p.add_argument('-d', '--debug-dvbs2', action='count', default=0,
+                        help="Debug leandvb's DVB-S2 decoding. Use it multiple "
                         "times to increase the debugging level")
     ldvb_p.add_argument('-v', '--verbose', default=False, action='store_true',
                         help='leandvb in verbose mode')
@@ -73,9 +177,10 @@ def subparser(subparsers):
                         help='leandvb fast lock mode')
     ldvb_p.add_argument('--rrc-rej', default=30, type=int,
                         help='leandvb RRC rej parameter')
-    ldvb_p.add_argument('-m', '--modcod', default="low",
-                        choices=["low", "high"],
-                        help="Choose low-throughput vs high-throughput MODCOD")
+    ldvb_p.add_argument('-m', '--modcod', choices=defs.modcods.keys(),
+                        default='qpsk3/5', metavar='',
+                        help="DVB-S2 modulation and coding (MODCOD) scheme. "
+                        "Choose from: " + ", ".join(defs.modcods.keys()))
     ldvb_p.add_argument('--ldpc-dec', default="ext", choices=["int", "ext"],
                         help="LDPC decoder to use (internal or external)")
     ldvb_p.add_argument('--ldpc-bf', default=100,
@@ -93,32 +198,9 @@ def subparser(subparsers):
                         help='Size in Mbytes of the input pipe file read by \
                         leandvb')
 
-    tsp_p = p.add_argument_group('tsduck options')
-    tsp_p.add_argument('--buffer-size-mb', default=1.0, type=float,
-                       help='Input buffer size in MB')
-    tsp_p.add_argument('--max-flushed-packets', default=10, type=int,
-                       help='Maximum number of packets processed by a tsp '
-                       'processor')
-    tsp_p.add_argument('--max-input-packets', default=10, type=int,
-                       help='Maximum number of packets received at a time from '
-                       'the tsp input plugin ')
-    tsp_p.add_argument('-p', '--bitrate-period', default=1, type=int,
-                       help='Period of bitrate reports in seconds')
-    tsp_p.add_argument('-l', '--local-address', default="127.0.0.1",
-                       help='IP address of the local interface on which to '
-                       'listen for UDP datagrams')
-    tsp_p.add_argument('-a', '--analyze', default=False, action='store_true',
-                       help='Analyze transport stream and save report on '
-                       'program termination')
-    tsp_p.add_argument('--analyze-file', default="ts-analysis.txt",
-                       action='store_true',
-                       help='File on which to save the MPEG-TS analysis.')
-    tsp_p.add_argument('--no-bitrate-monitoring', default=False,
-                       action='store_true',
-                       help='Disable bitrate monitoring')
-    tsp_p.add_argument('--monitor-ts', default=False,
-                       action='store_true',
-                       help='Mnitor MPEG TS sequence discontinuities')
+    # TSDuck Options
+    tsp.add_to_parser(p)
+
     p.set_defaults(func=run,
                    record=False)
 
@@ -159,10 +241,9 @@ def run(args):
 
     # Receiver configs
     l_band_freq = info['freqs']['l_band']*1e6
-    modcod      = defs.low_rate_modcod if args.modcod == "low" else \
-                  defs.high_rate_modcod
-    sym_rate    = defs.sym_rate[info['sat']['alias']]
-    samp_rate   = args.sps*sym_rate
+    sym_rate = defs.sym_rate[info['sat']['alias']]
+    samp_rate = args.sps*sym_rate
+    modcod = str(2**defs.modcods[args.modcod])  # bit mask
 
     assert(samp_rate < 2.4e6), \
         "Sample rate of {} exceeds the RTL-SDR limit".format(samp_rate)
@@ -184,26 +265,26 @@ def run(args):
                    "-s", str(samp_rate), "-d", str(args.rtl_idx)]
         if (args.record):
             bytes_per_sec = (2*samp_rate)/(2**20)
-            print("IQ recording will be saved on file {}".format(
+            logger.info("IQ recording will be saved on file {}".format(
                 args.iq_file))
-            print(textwrap.fill("NOTE: the file will grow by approximately "
-                                "{} MB per second.".format(bytes_per_sec)))
+            logger.info(
+                textwrap.fill("NOTE: the file will grow by approximately "
+                              "{} MB per second.".format(bytes_per_sec)))
             if (not util._ask_yes_or_no("Proceed?", default="y")):
                 return
             rtl_cmd.append(args.iq_file)
         else:
             rtl_cmd.append("-")
 
-    ldvb_cmd  = ["leandvb", "--nhelpers", str(args.n_helpers), "-f",
+    ldvb_cmd = ["leandvb", "--nhelpers", str(args.n_helpers), "-f",
                 str(samp_rate), "--sr", str(sym_rate), "--roll-off",
                 str(defs.rolloff), "--standard", "DVB-S2", "--sampler", "rrc",
                 "--rrc-rej", str(args.rrc_rej), "--modcods", modcod,
-                "--framesizes", str(args.framesizes), "--inpipe",
-                str(pipe_size_bytes)]
-    if (args.debug_ts == 1):
-        ldvb_cmd.append("-d")
-    elif (args.debug_ts > 1):
-        ldvb_cmd.extend(["-d", "-d"])
+                "--framesizes", str(args.framesizes)]
+    if (args.iq_file is None):
+        ldvb_cmd.extend(["--inpipe", str(pipe_size_bytes)])
+    if (args.debug_dvbs2 > 0):
+        ldvb_cmd.extend(("-d " * args.debug_dvbs2).split())
     if (args.ldpc_dec == "ext"):
         ldvb_cmd.extend(["--ldpc-helper", which("ldpc_tool"),
                          "--ldpc-iterations", str(args.ldpc_iterations)])
@@ -218,37 +299,11 @@ def run(args):
     if (derotate != 0.0):
         ldvb_cmd.extend(["--derotate", str(int(derotate))])
 
-    ldvb_stderr = None if (args.debug_ts > 0) else subprocess.DEVNULL
-
-    # Input
-    tsp_cmd  = ["tsp", "--realtime", "--buffer-size-mb",
-                str(args.buffer_size_mb), "--max-flushed-packets",
-                str(args.max_flushed_packets), "--max-input-packets",
-                str(args.max_input_packets)]
-    # MPEG-TS Analyzer
-    if (args.analyze):
-        print("MPEG-TS analysis will be saved on file {}".format(
-            args.analyze_file))
-        if (not util._ask_yes_or_no("Proceed?", default="y")):
-            return
-        tsp_cmd.extend(["-P", "analyze", "-o", args.analyze_file])
-    if (not args.no_bitrate_monitoring):
-        # Monitor Bitrate
-        tsp_cmd.extend(["-P", "bitrate_monitor", "-p",
-                        str(args.bitrate_period), "--min", "0"])
-    if (args.monitor_ts):
-        # Monitor MPEG TS discontinuities
-        tsp_cmd.extend(["-P", "continuity"])
-
-    # MPE plugin
-    tsp_cmd.extend(["-P", "mpe", "--pid",
-                    "-".join([str(pid) for pid in defs.pids]), "--udp-forward",
-                    "--local-address", args.local_address])
-    # Output
-    tsp_cmd.extend(["-O", "drop"])
+    ldvb_stderr = None if (args.debug_dvbs2 > 0) else subprocess.DEVNULL
 
     logger.debug("Run:")
 
+    # If recording IQ samples, run the rtl_sdr only
     if (args.record):
         p1 = subprocess.Popen(rtl_cmd)
         try:
@@ -256,26 +311,57 @@ def run(args):
         except KeyboardInterrupt:
             p1.kill()
         return
-    elif (args.iq_file is None):
+
+    # Create unnamed pipe to receive real-time demodulation information from
+    # leandvb (via --fd-info). Collect this information on a daemon thread.
+    if (args.iq_file is None):
+        info_pipe = util.Pipe()
+        fd_info   = str(info_pipe.w_fd)
+        ldvb_cmd.extend(["--fd-info", fd_info])
+
+        monitor = _get_monitor(args)
+
+        t = threading.Thread(
+            target = _monitor_demod_info,
+            args   = (info_pipe, monitor),
+            daemon = True
+        )
+        t.start()
+
+    # Prepare and validate the tsp command
+    tsp_handler = tsp.Tsp()
+    if (not tsp_handler.gen_cmd(args)):
+        return
+
+    if (args.iq_file is None):
         full_cmd  = "> " + " ".join(rtl_cmd) + " | \\\n" + \
                     " ".join(ldvb_cmd)
         p1 = subprocess.Popen(rtl_cmd, stdout=subprocess.PIPE)
-        p2 = subprocess.Popen(ldvb_cmd, stdin=p1.stdout, stdout=subprocess.PIPE,
-                              stderr=ldvb_stderr)
+        p2 = subprocess.Popen(
+            ldvb_cmd,
+            stdin=p1.stdout,
+            stdout=subprocess.PIPE,
+            stderr=ldvb_stderr,
+            pass_fds=[fd_info]
+        )
     else:
         full_cmd   = "> " + " ".join(ldvb_cmd) + " < " + args.iq_file
         fd_iq_file = open(args.iq_file)
-        p2 = subprocess.Popen(ldvb_cmd, stdin=fd_iq_file,
-                              stdout=subprocess.PIPE,
-                              stderr=ldvb_stderr)
+        p2 = subprocess.Popen(
+            ldvb_cmd,
+            stdin=fd_iq_file,
+            stdout=subprocess.PIPE,
+            stderr=ldvb_stderr
+        )
+
     if (not args.no_tsp):
-        full_cmd += " | \\\n" + " ".join(tsp_cmd)
+        full_cmd += " | \\\n" + " ".join(tsp_handler.cmd)
         logger.debug(full_cmd)
-        p3 = subprocess.Popen(tsp_cmd, stdin=p2.stdout)
+        tsp_handler.run(stdin=p2.stdout)
         try:
-            p3.communicate()
+            tsp_handler.proc.communicate()
         except KeyboardInterrupt:
-            p3.kill()
+            tsp_handler.proc.kill()
             p2.kill()
             p1.kill()
     else:
@@ -286,4 +372,4 @@ def run(args):
             p2.kill()
             p1.kill()
 
-
+    print()
