@@ -3,22 +3,11 @@ from pprint import pformat
 from ipaddress import IPv4Interface
 import os, sys, signal, argparse, subprocess, time, logging, threading, json
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
-from . import config, util, defs, rp, firewall, ip, dependencies
+from . import config, util, defs, rp, firewall, ip, dependencies, monitoring
 import textwrap
 logger = logging.getLogger(__name__)
 
 
-def _setup_logfile(cfg_dir):
-    """Setup directory and file for dvbv5-zap logs"""
-    log_dir = os.path.join(cfg_dir, "logs")
-    if not os.path.exists(log_dir):
-        os.makedirs(log_dir)
-
-    name    = "usb-" + time.strftime("%Y%m%d-%H%M%S") + ".log"
-    logfile = os.path.join(log_dir, name)
-    return logfile
-
-
 def _find_v4l_lnb(info):
     """Find suitable LNB within v4l-utils preset LNBs
 
@@ -47,8 +36,9 @@ def _find_v4l_lnb(info):
 
         if (info['lnb']['pol'].lower() == "dual" and # user has dual-pol LNB
             (info['sat']['pol'] == "H" or # and satellite requires H pol
-             (info['lnb']['v1_pointed'] and # or LNB already operates with H pol
-              info['lnb']['v1_psu_voltage'] >= 16)
+             ('v1_pointed' in info['lnb'] and
+              info['lnb']['v1_pointed'] and
+              info['lnb']['v1_psu_voltage'] >= 16) # or LNB already operates with H pol
             ) and "rangeswitch" not in lnb): # but LNB candidate is single-pol
             continue # not a validate candidate, skip
 
@@ -67,28 +57,20 @@ def _find_v4l_lnb(info):
     return options[0]
 
 
-def _find_adapter(list_only=False):
+def _find_adapter(list_only=False, target_model=None):
     """Find the DVB adapter
 
     Returns:
         Tuple with (adapter index, frontend index)
 
     """
-    util._print_header("Find DVB Adapter")
-    ps     = subprocess.Popen("dmesg", stdout=subprocess.PIPE)
-    try:
-        output = subprocess.check_output(["grep", "frontend"], stdin=ps.stdout,
-                                         stderr=subprocess.STDOUT)
-    except subprocess.CalledProcessError as grepexc:
-        if (grepexc.returncode == 1):
-            output = ""
-            pass
-    ps.wait()
+    if (target_model is None):
+        util._print_header("Find DVB Adapter")
 
     # Search a range of adapters. There is no command to list all adapters, so
     # we try to list each one individually using `dvbnet -a adapter_no -l`.
     adapters = list()
-    for a in range(0,10):
+    for a in range(0, 10):
         cmd     = ["dvbnet", "-a", str(a), "-l"]
         logger.debug("> " + " ".join(cmd))
 
@@ -115,9 +97,20 @@ def _find_adapter(list_only=False):
 
     # If nothing was obtained using dvbnet, try to inspect dmesg logs
     if (len(adapters) == 0):
-        lines       = output.splitlines()
-        adapter_set = set() # use set to filter unique values
-        adapters    = list()
+        ps = subprocess.Popen("dmesg", stdout=subprocess.PIPE)
+        try:
+            output = subprocess.check_output(["grep", "frontend"],
+                                             stdin=ps.stdout,
+                                             stderr=subprocess.STDOUT)
+        except subprocess.CalledProcessError as grepexc:
+            if (grepexc.returncode == 1):
+                output = ""
+                pass
+        ps.wait()
+
+        lines = output.splitlines()
+        adapter_set = set()  # use set to filter unique values
+        adapters = list()
         for line in lines:
             linesplit = line.decode().split()
             if ("adapter" not in linesplit):
@@ -155,23 +148,35 @@ def _find_adapter(list_only=False):
     assert(len(dvb_s2_adapters) > 0), "No DVB-S2 adapters found"
 
     chosen_adapter = None
-    for adapter in dvb_s2_adapters:
-        print("Found DVB-S2 adapter: %s %s" %(adapter["vendor"],
-                                              adapter["model"]))
-
-        if (not list_only):
-            if (len(dvb_s2_adapters) == 1 or
-                util._ask_yes_or_no("Choose adapter?")):
+    if (target_model is not None):
+        # Search without prompting
+        for adapter in dvb_s2_adapters:
+            if (adapter["model"] == target_model):
                 chosen_adapter = adapter
-                logger.debug("Chosen adapter:")
-                logger.debug(pformat(adapter))
                 break
+    else:
+        # Prompt the user
+        for adapter in dvb_s2_adapters:
+            print("Found DVB-S2 adapter: %s %s" %(adapter["vendor"],
+                                                  adapter["model"]))
+
+            if (not list_only):
+                if (len(dvb_s2_adapters) == 1 or
+                    util._ask_yes_or_no("Choose adapter?")):
+                    chosen_adapter = adapter
+                    logger.debug("Chosen adapter:")
+                    logger.debug(pformat(adapter))
+                    break
 
     if (list_only):
         return
 
     if (chosen_adapter is None):
-        raise ValueError("Please choose DVB-S2 adapter")
+        if (target_model is None):
+            err_msg = "Please choose a DVB-S2 adapter"
+        else:
+            err_msg = "Please choose a valid DVB-S2 adapter"
+        raise ValueError(err_msg)
 
     return chosen_adapter["adapter"], chosen_adapter["frontend"]
 
@@ -346,7 +351,7 @@ def _rm_dvbnet_interface(adapter, ifname, verbose=True):
 
 
 def zap(adapter, frontend, ch_conf_file, user_info, lnb="UNIVERSAL",
-        output=None, timeout=None, monitor=False, scrolling=False):
+        output=None, timeout=None, monitor=False):
     """Run zapper
 
     Args:
@@ -359,26 +364,30 @@ def zap(adapter, frontend, ch_conf_file, user_info, lnb="UNIVERSAL",
         timeout      : Run the zap for this specified duration
         monitor      : Monitor mode. Monitors DVB traffic stats (throughput and
                        packets per second), but does not deliver data upstream.
-        scrolling    : Whether to print zap logs by scrolling rather than
-                       printing always on the same line.
 
     Returns:
         Subprocess object
 
     """
 
-    util._print_header("Tuning DVB Receiver")
-    print("Running dvbv5-zap")
+    util._print_header("Tuning the DVB-S2 Receiver")
 
     # LNB name to use when calling dvbv5-zap
     if (lnb is None):
         # Find suitable LNB within v4l-utils preset LNBs
         lnb = _find_v4l_lnb(user_info)['alias']
 
+    print("Running dvbv5-zap")
+    print("  - Config:   {}".format(ch_conf_file))
+    print("  - Adapter:  {}".format(adapter))
+    print("  - Frontend: {}".format(frontend))
+    print("  - LNB:      {}".format(lnb))
+
     cmd = ["dvbv5-zap", "-c", ch_conf_file, "-a", str(adapter), "-f",
            str(frontend), "-l", lnb, "-v"]
 
-    if (output is not None):
+    rec_mode = output is not None
+    if (rec_mode):
         cmd = cmd + ["-o", output]
 
         if (os.path.exists(output)):
@@ -406,8 +415,6 @@ def zap(adapter, frontend, ch_conf_file, user_info, lnb="UNIVERSAL",
         cmd = cmd + ["-t", timeout]
 
     if (monitor):
-        assert(not scrolling), \
-            "Monitor mode does not work with scrolling (line-by-line) logs"
         assert(output is None), \
             "Monitor mode does not work if recording (i.e. w/ -r flag)"
         cmd.append("-m")
@@ -416,12 +423,11 @@ def zap(adapter, frontend, ch_conf_file, user_info, lnb="UNIVERSAL",
 
     logger.debug("> " + " ".join(cmd))
 
-    if (scrolling):
+    if (monitor or rec_mode):
+        ps = subprocess.Popen(cmd)
+    else:
         ps = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE, universal_newlines=True)
-    else:
-        ps = subprocess.Popen(cmd)
-
     return ps
 
 
@@ -435,7 +441,7 @@ def subparser(subparsers):
     # Common parameters
     p.add_argument('-a', '--adapter',
                    default=None,
-                   help='DVB-S2 adapter number')
+                   help='DVB-S2 adapter number or model')
 
     p.add_argument('-f', '--frontend',
                    default=None,
@@ -449,35 +455,30 @@ def subparser(subparsers):
     # Launch
     p1 = subsubparsers.add_parser('launch',
                                   description="Launch a USB DVB-S2 receiver",
-                                  help='Launch a Linux USB DVB-S2 receiver')
+                                  help='Launch a Linux USB DVB-S2 receiver',
+                                  formatter_class=ArgumentDefaultsHelpFormatter)
 
     p1.add_argument('-l', '--lnb',
                     choices=defs.lnb_options,
                     default=None,
-                    help="LNB from v4l-utils to be used. "
-                    "If None, i.e. not specified, it will be set "
-                    "automatically")
+                    metavar='',
+                    help="LNB being used. If not specified, it will be defined "
+                    "automatically. Choose from the following LNBs supported "
+                    "by v4l-utils: " + ", ".join(defs.lnb_options))
 
     p1.add_argument('-r', '--record-file', default=None,
-                    help='Record MPEG-TS traffic into target file')
+                    help='Record MPEG-TS traffic into a target file')
 
     p1.add_argument('-t', '--timeout', default=None,
-                    help='Stop zapping after timeout - useful to \
+                    help='Stop zapping after a timeout in seconds - useful to \
                     control recording time')
 
     p1.add_argument('-m', '--monitor', default=False,
                     action='store_true',
-                    help='Launch dvbv5-zap in monitor mode - useful \
-                    to debug packet and bit rates')
+                    help='Launch dvbv5-zap in monitor mode to debug MPEG TS '
+                    'packet and bit rates')
 
-    p1.add_argument('-s', '--scrolling', default=False,
-                    action='store_true',
-                    help='Print dvbv5-zap logs line-by-line, i.e. \
-                    scrolling, rather than always on the same line')
-
-    p1.add_argument('--logfile', default=False,
-                    action='store_true',
-                    help='Save dvbv5-zap logs on a file')
+    monitoring.add_to_parser(p1)
 
     p1.set_defaults(func=launch)
 
@@ -485,7 +486,8 @@ def subparser(subparsers):
     p2 = subsubparsers.add_parser('config', aliases=['cfg'],
                                   description='Initial configurations',
                                   help='Configure DVB-S2 interface(s) and\
-                                  the host')
+                                  the host',
+                                  formatter_class=ArgumentDefaultsHelpFormatter)
     p2.add_argument('-U', '--ule', default=False,
                     action='store_true',
                     help='Use ULE encapsulation instead of MPE')
@@ -525,9 +527,10 @@ def subparser(subparsers):
                                   description="Remove DVB-S2 adapter",
                                   help='Remove DVB-S2 adapter',
                                   formatter_class=ArgumentDefaultsHelpFormatter)
-    p4.add_argument('-a', '--adapter',
-                           default=None,
-                           help='DVB-S2 adapter number')
+    p4.add_argument('--all', default=False,
+                    action='store_true',
+                    help='Remove all dvbnet interfaces associated with the '
+                    'adapter')
     p4.set_defaults(func=rm_subcommand)
 
     return p
@@ -548,8 +551,22 @@ def _common(args):
     if (args.adapter is None):
         adapter, frontend = _find_adapter()
     else:
-        adapter  = args.adapter
-        frontend = args.frontend
+        if (args.adapter.isdigit()):
+            # Assume argument --adapter has the adapter number. In this case,
+            # --frontend is also required.
+            if (args.frontend is None):
+                raise ValueError("argument --adapter requires --frontend.")
+            adapter  = args.adapter
+            frontend = args.frontend
+        else:
+            # Assume argument --adapter has the target model
+            adapter, frontend = _find_adapter(target_model=args.adapter)
+
+    # Cache the adapter number on the local config file so that other modules
+    # can read the adapter number directly. Rewrite the info every time, as the
+    # adapter number could change between boots.
+    user_info['setup']['adapter'] = adapter
+    config.write_cfg_file(args.cfg, args.cfg_dir, user_info)
 
     return user_info, adapter, frontend
 
@@ -604,7 +621,8 @@ def usb_config(args):
 
     # Set firewall rules
     if (not args.skip_firewall):
-        firewall.configure(net_ifs, defs.src_ports, prompt=(not args.yes))
+        firewall.configure(net_ifs, defs.src_ports, user_info['sat']['ip'],
+                           prompt=(not args.yes))
 
     # Set IP
     ip.set_ips(net_ifs, ips)
@@ -615,6 +633,62 @@ def usb_config(args):
     return
 
 
+log_key_map = {
+    'Lock'    : 'lock',
+    'Signal'  : 'level',
+    'C/N'     : 'snr',
+    'postBER' : 'ber'
+}
+
+
+def _parse_log(line):
+    """Parse logs from dvbv5-zap and convert to format accepted by monitor.py"""
+    if (line is None or line == "\n"):
+        return
+
+    # Don't process the "Layer A" lines
+    if ("Layer" in line):
+        return
+
+    # Only process lines containing "Signal"
+    if ("Signal" not in line):
+        return
+    d           = {}
+    elements    = line.split()
+    n_elem      = len(elements)
+    d['lock']   = ("Lock" in line, None)
+    for i, elem in enumerate(elements):
+        # Each metric is printed as "name=value"
+        if (elem[-1] == "=" and (i+1) <= n_elem):
+            key       = elem[:-1]
+            raw_value = elements[i+1]
+
+            # Convert key to the nomenclature adopted on monitor.py
+            key = log_key_map[key]
+
+            # Fix any scientific notation
+            if ("^" in raw_value):
+                raw_value = raw_value.replace("x10^","e")
+
+            # Parse value and unit
+            unit = None
+            if ("%" in raw_value):
+                unit = "%"
+                val  = float(raw_value[:-1].replace(",","."))
+            elif (raw_value[-2:] == "dB"):
+                val  = float(raw_value[:-2].replace(",","."))
+                unit = "dB"
+            elif (raw_value[-3:] == "dBm"):
+                val  = float(raw_value[:-3].replace(",","."))
+                unit = "dBm"
+            else:
+                val = float(raw_value.replace(",", "."))
+
+            d[key] = (val, unit)
+
+    return d
+
+
 def launch(args):
     """Launch the DVB interface from scratch
 
@@ -631,17 +705,32 @@ def launch(args):
     if (not dependencies.check_apps(["dvbv5-zap"])):
         return
 
+    # Check conflicting logging options
+    if ((args.monitor or args.record_file) and
+        (args.log_scrolling or args.log_file)):
+        raise ValueError("Logging options are disabled when running with "
+                         "-m/--monitor or -r/--record-file")
+
+    # Log Monitoring
+    monitor = monitoring.Monitor(
+        args.cfg_dir,
+        logfile = args.log_file,
+        scroll = args.log_scrolling,
+        min_interval = args.log_interval,
+        server = args.monitoring_server,
+        port = args.monitoring_port,
+        report = args.report,
+        report_opts = monitoring.get_report_opts(args),
+        utc = args.utc
+    )
+
     # Channel configuration file
     chan_conf = user_info['setup']['channel']
 
     # Zap
     zap_ps = zap(adapter, frontend, chan_conf, user_info, lnb=args.lnb,
                  output=args.record_file, timeout=args.timeout,
-                 monitor=args.monitor, scrolling=(args.scrolling
-                                                  or args.logfile))
-
-    # Prepare logs
-    logfile = None if (not args.logfile) else _setup_logfile(args.cfg_dir)
+                 monitor=args.monitor)
 
     # Handler for SIGINT
     def signal_handler(sig, frame):
@@ -652,30 +741,20 @@ def launch(args):
     signal.signal(signal.SIGINT, signal_handler)
     signal.signal(signal.SIGTERM, signal_handler)
 
+    util._print_header("Receiver Monitoring")
+    print()
+
     # Listen to dvbv5-zap indefinitely
-    if (args.scrolling or args.logfile):
-        # Loop indefinitely over zap
-        prev_line = None
-        while (zap_ps.poll() is None):
-            line = zap_ps.stderr.readline()
-            if (line and line != "\n"):
-                pretty_line = '\r{}: '.format(
-                    time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())) + line
-                if (("Signal" in line) and ("Layer" not in line)):
-                    prev_line = pretty_line.replace("\n", " ")
-                else:
-                    concat_line = pretty_line if prev_line is None else \
-                                  (prev_line + line)
-                    final_line  = " ".join(concat_line.split()) + "\n"
-                    prev_line   = None
-                    print(final_line, end='')
-                    if (logfile is not None):
-                        with open(logfile, 'a') as fd:
-                            fd.write(final_line)
-            else:
-                time.sleep(1)
-    else:
+    if (args.monitor or args.record_file):
         zap_ps.wait()
+    else:
+        while (zap_ps.poll() is None):
+            line    = zap_ps.stderr.readline()
+            metrics = _parse_log(line)
+            if (metrics is None):
+                continue
+            monitor.update(metrics)
+    print()
     sys.exit(zap_ps.poll())
 
 
@@ -706,33 +785,42 @@ def rm_subcommand(args):
 
     # Find adapter
     if (args.adapter is None):
-        adapter, frontend = _find_adapter()
+        adapter, _ = _find_adapter()
     else:
-        adapter = args.adapter
+        # If argument --adapter holds a digit-only string, assume it refers to
+        # the adapter number. Otherwise, assume it refers to the target model.
+        adapter = args.adapter if args.adapter.isdigit() else \
+            _find_adapter(target_model=args.adapter)[0]
 
     interfaces     = _find_dvbnet_interfaces(adapter)
     chosen_devices = list()
 
     if (len(interfaces) > 1):
-        print("Choose net device to remove:")
-        for i_dev, interface in enumerate(interfaces):
-            print("[%2u] %s" %(i_dev, interface['name']))
-        print("[ *] all")
+        if (args.all):
+            for interface in interfaces:
+                chosen_devices.append(interface['name'])
+        else:
+            print("Choose net device to remove:")
+            for i_dev, interface in enumerate(interfaces):
+                print("[%2u] %s" %(i_dev, interface['name']))
+            print("[ *] all")
 
-        try:
-            choice = input("Choose number: ")
-            if (choice == "*"):
-                i_chosen_devices = range(0, len(interfaces))
-            else:
-                i_chosen_devices = [int(choice)]
-        except ValueError:
-            raise ValueError("Please choose a number or \"*\" for all devices")
+            try:
+                choice = input("Choose number: ")
+                if (choice == "*"):
+                    i_chosen_devices = range(0, len(interfaces))
+                else:
+                    i_chosen_devices = [int(choice)]
+            except ValueError:
+                raise ValueError(
+                    "Please choose a number or \"*\" for all devices"
+                )
 
-        for i_chosen_dev in i_chosen_devices:
-            if (i_chosen_dev > len(interfaces)):
-                raise ValueError("Invalid number")
+            for i_chosen_dev in i_chosen_devices:
+                if (i_chosen_dev > len(interfaces)):
+                    raise ValueError("Invalid number")
 
-            chosen_devices.append(interfaces[i_chosen_dev]['name'])
+                chosen_devices.append(interfaces[i_chosen_dev]['name'])
 
     elif (len(interfaces) == 0):
         print("No DVB network interfaces to remove")
@@ -742,10 +830,6 @@ def rm_subcommand(args):
         chosen_devices.append(interfaces[0]['name'])
 
     for chosen_dev in chosen_devices:
-        if (not util._ask_yes_or_no("Remove interface %s?" %(chosen_dev))):
-            print("Aborting...")
-            return
-
         _rm_dvbnet_interface(adapter, chosen_dev)
 
 
