@@ -1,5 +1,5 @@
 """Utility functions"""
-import os, textwrap, subprocess
+import copy, os, textwrap, subprocess
 
 
 def fill_print(text):
@@ -8,12 +8,17 @@ def fill_print(text):
     print()
 
 
-def typed_input(msg, hint=None, in_type=int):
+def typed_input(msg, hint=None, in_type=int, default=None):
     """Ask for user input of a specific type"""
     res = None
     while (res is None):
         try:
-            res = in_type(input(msg + ": "))
+            if (default is not None):
+                assert(isinstance(default, in_type))
+                input_val = input(msg + ": [{}] ".format(default)) or default
+            else:
+                input_val = input(msg + ": ")
+            res = in_type(input_val)
         except ValueError:
             if (hint is None):
                 if (in_type == int):
@@ -121,7 +126,9 @@ def _ask_multiple_choice(vec, msg, label, to_str, help_msg = None,
             choice = None
             print(none_str)
         else:
-            choice = vec[resp]
+            choice = copy.deepcopy(vec[resp])
+            # NOTE: deep copy to prevent the caller from changing the original
+            # element.
             print(to_str(choice))
         print()
 
@@ -220,6 +227,19 @@ class ProcessRunner():
         self.dry      = dry
         self.last_cwd = None
 
+    def _get_cmd_str(self, orig_cmd):
+        """Generate string for command
+
+        If any argument is supposed to be quoted (i.e., has spaces), add the
+        quotes.
+
+        """
+        quoted_cmd = orig_cmd.copy()
+        for i, elem in enumerate(quoted_cmd):
+            if (" " in elem):
+                quoted_cmd[i] = "\'{}\'".format(elem)
+        return " ".join(quoted_cmd)
+
     def run(self, cmd, cwd=None, env=None, stdout=None, stderr=None,
             nocheck=False):
         assert(isinstance(cmd, list))
@@ -227,11 +247,11 @@ class ProcessRunner():
             if (cwd != self.last_cwd):
                 print("cd {}".format(cwd))
                 self.last_cwd = cwd
-            print(" ".join(cmd))
+            print(self._get_cmd_str(cmd))
             return
 
         if (self.logger is not None):
-            self.logger.debug("> " + " ".join(cmd))
+            self.logger.debug("> " + self._get_cmd_str(cmd))
 
         res = subprocess.run(cmd, cwd=cwd, env=env, stdout=stdout,
                              stderr=stderr)
@@ -240,3 +260,34 @@ class ProcessRunner():
         return res
 
 
+class Pipe():
+    """Unnamed pipe wrapper"""
+    def __init__(self):
+        """Create unnamed pipe"""
+        r_fd, w_fd = os.pipe()
+
+        self.r_fd  = r_fd                  # read file descriptor
+        self.r_fo  = os.fdopen(r_fd, "r")  # read file object
+
+        self.w_fd  = w_fd                  # write file descriptor
+        self.w_fo  = os.fdopen(w_fd, "w")  # write file object
+
+    def __del__(self):
+        """Close pipe"""
+        self.r_fo.close()
+        self.w_fo.close()
+
+    def readline(self):
+        """Read line from pipe file"""
+        return self.r_fo.readline()
+
+    def write(self, data):
+        """Write data to pipe file
+
+        Args:
+            data : The data to write into the pipe file
+
+        """
+        self.w_fo.write(data)
+
+
